@@ -3,7 +3,7 @@ from magento.version import __version__
 from magento.exceptions import MagentoException
 from magento.attributes import (
     get_custom_attribute, get_boolean_custom_attribute, get_custom_attributes_dict,
-    pretty_custom_attributes
+    pretty_custom_attributes, set_custom_attribute
 )
 from magento.order_helpers import (
     is_order_on_hold, is_order_cash_on_delivery, get_order_shipping_address
